#include "ImGuiLayer.hpp"

// TODO: Remove these, these are temporary
#include <glad/gl.h>
#include <GLFW/glfw3.h>

#include <imgui.h>
#include <backends/imgui_impl_opengl3.h>
#include <quark.hpp>

#include "platform/IoTranslate.hpp"
#include "quark/Application.hpp"
#include "quark/events/Event.hpp"

namespace qk {
	ImGuiLayer::ImGuiLayer() :
		Layer("ImGui Quark")
	{
		
	}

	ImGuiLayer::~ImGuiLayer() {
		
	}

	void ImGuiLayer::OnAttach() {
		ImGui::CreateContext();
		ImGui::StyleColorsDark();

		ImGuiIO& io = ImGui::GetIO();
		Application& app = Application::GetInstance();

		io.DisplaySize = ImVec2(app.GetWindow().GetWidth(), app.GetWindow().GetHeight());

		io.BackendFlags |= ImGuiBackendFlags_HasMouseCursors;
		io.BackendFlags |= ImGuiBackendFlags_HasSetMousePos;
		// io.BackendFlags |= ImGuiBackendFlags_PlatformHasViewports;
		// io.BackendFlags |= ImGuiBackendFlags_RendererHasViewports;

		io.ConfigFlags |= ImGuiConfigFlags_DockingEnable;
		// io.ConfigFlags |= ImGuiConfigFlags_ViewportsEnable;

		ImGui_ImplOpenGL3_Init("#version 410");
	}

	void ImGuiLayer::OnDetach() {
		
	}

	void ImGuiLayer::OnUpdate() {
		ImGui_ImplOpenGL3_NewFrame();
		ImGui::NewFrame();

		float time = (float)glfwGetTime();
		ImGui::GetIO().DeltaTime = (lastTime > 0.0f) ? (time - lastTime) : (1.0f / 60.0f);
		lastTime = time;

		static bool show = true;
		ImGui::ShowDemoWindow(&show);

		ImGui::Render();
		ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
	}
	
	void ImGuiLayer::OnEvent(Event& event) {
		EventDispatcher dispatcher(event);

		dispatcher.Dispatch<MouseButtonPressedEvent>(BIND_EVENT_CB(ImGuiLayer::OnMouseButtonPressed));
		dispatcher.Dispatch<MouseButtonReleasedEvent>(BIND_EVENT_CB(ImGuiLayer::OnMouseButtonReleased));
		dispatcher.Dispatch<MouseMovedEvent>(BIND_EVENT_CB(ImGuiLayer::OnMouseMoved));
		dispatcher.Dispatch<MouseScrolledEvent>(BIND_EVENT_CB(ImGuiLayer::OnMouseScrolled));

		dispatcher.Dispatch<KeyPressedEvent>(BIND_EVENT_CB(ImGuiLayer::OnKeyPressedEvent));
		dispatcher.Dispatch<KeyReleasedEvent>(BIND_EVENT_CB(ImGuiLayer::OnKeyReleasedEvent));
		dispatcher.Dispatch<KeyTypedEvent>(BIND_EVENT_CB(ImGuiLayer::OnKeyTypedEvent));

		dispatcher.Dispatch<WindowResizeEvent>(BIND_EVENT_CB(ImGuiLayer::OnWindowResized));
	}

	bool ImGuiLayer::OnMouseButtonPressed(MouseButtonPressedEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		io.AddMouseButtonEvent(TranslateMouseButton(e.GetMouseButton()), true);

		return false;
	}

	bool ImGuiLayer::OnMouseButtonReleased(MouseButtonReleasedEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		io.AddMouseButtonEvent(TranslateMouseButton(e.GetMouseButton()), false);

		return false;
	}

	bool ImGuiLayer::OnMouseMoved(MouseMovedEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		io.AddMousePosEvent(e.GetX(), e.GetY());

		return false;
	}

	bool ImGuiLayer::OnMouseScrolled(MouseScrolledEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		io.AddMouseWheelEvent(e.GetX(), e.GetY());

		return false;
	}

	bool ImGuiLayer::OnKeyPressedEvent(KeyPressedEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		ImGuiKey key = TranslateKey(e.GetKeyCode());

		if (key == ImGuiKey_LeftCtrl || key == ImGuiKey_RightCtrl) {
			io.AddKeyEvent(ImGuiMod_Ctrl, true);
		} 

		if (key == ImGuiKey_LeftShift || key == ImGuiKey_RightShift) {
			io.AddKeyEvent(ImGuiMod_Shift, true);
		} 

		if (key == ImGuiKey_LeftAlt || key == ImGuiKey_RightAlt) {
			io.AddKeyEvent(ImGuiMod_Alt, true);
		}

		if (key == ImGuiKey_LeftSuper || key == ImGuiKey_RightSuper) {
			io.AddKeyEvent(ImGuiMod_Super, true);
		}

		io.AddKeyEvent(key, true);

		return false;
	}

	bool ImGuiLayer::OnKeyReleasedEvent(KeyReleasedEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		ImGuiKey key = TranslateKey(e.GetKeyCode());

		if (key == ImGuiKey_LeftCtrl || key == ImGuiKey_RightCtrl) {
			io.AddKeyEvent(ImGuiMod_Ctrl, false);
		} 

		if (key == ImGuiKey_LeftShift || key == ImGuiKey_RightShift) {
			io.AddKeyEvent(ImGuiMod_Shift, false);
		} 

		if (key == ImGuiKey_LeftAlt || key == ImGuiKey_RightAlt) {
			io.AddKeyEvent(ImGuiMod_Alt, false);
		}

		if (key == ImGuiKey_LeftSuper || key == ImGuiKey_RightSuper) {
			io.AddKeyEvent(ImGuiMod_Super, false);
		}

		io.AddKeyEvent(key, false);

		return false;
	}

	bool ImGuiLayer::OnKeyTypedEvent(KeyTypedEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		int c = e.GetKeyCode();

		if (c > 0 && c < 0x10000) {
			io.AddInputCharacter(c);
		}

		return false;
	}

	bool ImGuiLayer::OnWindowResized(WindowResizeEvent& e) {
		ImGuiIO& io = ImGui::GetIO();
		io.DisplaySize = ImVec2(e.GetWidth(), e.GetHeight());
		io.DisplayFramebufferScale = ImVec2(1.f, 1.f);

		glViewport(0, 0, e.GetWidth(), e.GetHeight());

		return false;
	}

}
